 /*
    * Copyright (c) 2012, H. J. Ohlbach. All rights reserved.
    * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
    *
    * This code is free software; you can redistribute it and/or modify it
    * under the terms of the GNU General Public License version 2 only, as
    * published by the Free Software Foundation. 
    *
    * This code is distributed in the hope that it will be useful, but WITHOUT
    * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    * version 2 for more details.
 -   */

package Java2OWL;

import java.util.ArrayList;
import java.util.List;

/** This is the top manager for the Java2OWL system. 
 * It contains all the other necessary managers.
 * 
 * One can create several instances of J2OManager, but all of them work with different 
 * ontologies which have nothing to do with each other. 
 * If extension ontologies are shared between different managers, anything can happen. 
 *
 * @author ohlbach
 */
public class J2OManager {
    
    /** stores all the currently active managers.*/
    private static List<J2OManager> managers = new ArrayList<J2OManager>();
    /** the name of the manager. */
    private String      name;    
    /** counts the managers. */
    private static int  globalCounter = 0;          
    /** the number of the manager. */
    private int         localNumber;   
    /** the corresponding ontology manager. */
    private J2OOntologyManager      ontologyManager     = null;
    /** the corresponding class manager. */
    private J2OClassManager         classManager        = null;
    /** the corresponding individual manager. */
    private J2OIndividualManager    indvidualManager    = null;
    /** the corresponding compiler.  */
    private J2OCompiler             compiler            = null;
    /** the corresponding logger.  */
    private Logging                 logger              = null;

    
    /** returns the name of the manager. */
    public String getName() {return name;}
    
    /** returns the index number of this manager.
     * This is used in the logger to distinguish between different managers.
     */
    public int getNumber() {return localNumber;}

    /** returns the class-manager. */
    public J2OClassManager getClassManager() {return classManager;}
    
    /** just for test purposes */
    void setClassManager(J2OClassManager clm) {classManager = clm;}

    /** returns the Java -> OWL compiler. */
    public J2OCompiler getCompiler() {return compiler;}

    /** returns the individual manager. */
    public J2OIndividualManager getIndvidualManager() {return indvidualManager;}

    /** returns the ontology manager. */
    public J2OOntologyManager getOntologyManager() {return ontologyManager;}
    
    /** returns the logger. */
    public Logging getLogger() {return logger;}
    
    /** returns all managers. */
    public static List<J2OManager> getManagers() {return managers;}
    
    /** The system can be in different statuses.*/
    public enum Status {
        /** No individual manager is necessary. */
        COMPILE,
        /** All managers are necessary. */
        RUN;}
    
    /** the status of the system. */
    private Status status;
    
    /** returns the status of the system. */
    public Status getStatus() {return status;}
    
    /** returns true if the status is COMPILE. */
    public boolean statusCompile() {return status == Status.COMPILE;}
    
    /** returns true if the status is RUN. */
    public boolean statusRun() {return status == Status.RUN;}
    
    
     /** indicates the debug mode. */
    private boolean debugMode = false;
    
    /** sets the debug mode. */
    public void setDebugMode(boolean flag) {
        debugMode = flag;
        logger.setDebugMode(flag);
    }
    
    /** returns true if the system is in debug mode */
    public boolean isInDebugMode() {return debugMode;}
    
    /** creates a manager with the given name and status.
     * All necessary managers are created 
     * @param name  The name of the system.
     * @param status The status of the system.
     */
    public J2OManager(String name, Status status, boolean debugMode) {
        this.name = name;
        this.status = status;
        this.debugMode = debugMode; 
        logger = new Logging(this);
        ontologyManager = new J2OOntologyManager(this);
        classManager = new J2OClassManager(this);
        ontologyManager.setClassManager(classManager);
        compiler = new J2OCompiler(this); 
        if(status == Status.RUN) indvidualManager = new J2OIndividualManager(this);
        localNumber = globalCounter;
        ++globalCounter;
        managers.add(this); 
    }
    
    
    /** creates a manager with the given name for the RUN status.
     * All necessary managers are created 
     * @param name  The name of the system.
     */
    public J2OManager(String name, boolean debugMode) {
        this.name = name;
        this.status = J2OManager.Status.RUN;
        this.debugMode = debugMode;
        logger = new Logging(this);
        ontologyManager = new J2OOntologyManager(this);
        classManager = new J2OClassManager(this);
        compiler = new J2OCompiler(this); 
        indvidualManager = new J2OIndividualManager(this);
        localNumber = globalCounter;
        ++globalCounter;
        managers.add(this); 
    }
    
    /** removes the manager from the internal list.*/
    @Override
    public void finalize() throws Throwable {
        super.finalize();
        managers.remove(this);}
    
    
}
